// BMPData.h : interface of the CBMPData class
//
/////////////////////////////////////////////////////////////////////////////

#include <afxtempl.h>
#include <queue>
using namespace std;

#if !defined(AFX_BMPDATA_H__FFC3A527_E06A_4F89_83DA_449E3C851C65__INCLUDED_)
#define AFX_BMPDATA_H__FFC3A527_E06A_4F89_83DA_449E3C851C65__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#define STRATEGY_SCATTER_SEARCH		1
#define STRATEGY_GENETIC_ALGORITHM	2

int FindObIndexFromList(CObList& list, CObject* ob);

class CBMPSite;
class CLandUse;
class CAquifer;
class CPUMP;
class CFtable;

typedef struct tag_WEATHERDATA {
	COleDateTime tDATE;	// date for the weather file data
	bool	bWetInt;	// wet interval (wet day or with in 72 hrs after wet day)
	double	lfPrec;		// precip (in/timestep)
	double	lfDailyPrec;// Daily precip (in/day)
} WEATHERDATA;

typedef struct tag_BINDATA {
	COleDateTime tDATE;	// date for the precip bin file
	int	nBinID;			// unique bin id (serial number starting with 1)
} BINDATA;

typedef struct tag_POLLUTANT {
	int	m_nID;				//unique id
	CString	m_sName;		//name description
	int	m_nSedfg;			//sediment class (sand, silt, clay or total)
	int m_nSedQual;			//sediment associated qual flag (0=no, 1=yes)
	double	m_lfMult;		//time series multiplier
	double m_lfsand_qfr;	//pollutant fraction associated with sand
	double m_lfsilt_qfr;	//pollutant fraction associated with silt
	double m_lfclay_qfr;	//pollutant fraction associated with clay
} POLLUTANT;

typedef struct tag_BMPCOST {
	int	m_nBMPClass;
	CString	m_strBMPType;
	double	m_lfCost;
} BMPCOST;

class CBMPData
{
public:
	CBMPData();
	virtual ~CBMPData();

// Attributes
public:
	// skip all messages and timeseries output if run in batch mode.
	int nRUN_BATCH;
	bool RUN_CECURVE;		// run simulation from the CE-CURVE
	int nStrategy;			// Optimization strategy -- Scatter Search or Genetic Algorithm
	CString strInputDir;
	CString strOutputDir;
	CString strMixLUFileName;
	CString strPreLUFileName;
	CString strError;
	int nLandSimulation;	// Land Simulation Option (0-External,1-Internal)
	int nWeatherFile;		// precipiation file flag (optional)
	int nLANDTimeStep;		// Land Simulation Time Step (1 - 60 minutes)
	int nBMPTimeStep;		// BMP Simulation Time Step (1 - 60 minutes)
	double lfCRRAT;			// Ratio of max velocity to mean velocity (1.0 or greater than 1)
	int nOutputTimeStep;	// Output Time Step -- 0 for timestep, 1 for hourly
	int nBIORETENTION;		// total number of BIORETENTION in class A
	int nWETPOND;			// total number of WETPOND in class A
	int nCISTERN;			// total number of CISTERN in class A
	int nDRYPOND;			// total number of DRYPOND in class A
	int nINFILTRATIONTRENCH;// total number of INFILTRATIONTRENCH in class A
	int nGREENROOF;			// total number of GREENROOF in class A
	int nPOROUSPAVEMENT;	// total number of POROUSPAVEMENT in class A
	int nRAINBARREL;		// total number of RAINBARREL in class A
	int nREGULATOR;			// total number of REGULATOR in class A
	int nSWALE;				// total number of SWALE in class B
	int nBUFFERSTRIP;		// total number of BUFFERSTRIP in class D
	int nAREABMP;			// total number of AREABMP in class E
	int nBMPtype;			// total unique bmp types (A or B or D or E)
	int nBMPA;				// total number of BMPs in class A  - pond type
	int nBMPB;				// total number of BMPs in class B  - swale
	int nBMPC;				// total number of BMPs in class C	- conduit (01-2005)
	int nBMPD;				// total number of BMPs in class D	- bufferstrip (06-2007)
	int nBMPE;				// total number of BMPs in class E	- areabmp (03-2011)
	int nAdjVariable;		// total number of adjustable variables
	int nEvalFactor;		// total number of evaluation factors
	int nRunOption;			// 0 -- No Optimization, 1 -- Minimize Cost, 2 -- Cost-Effective Curve, 3 -- Maximize Control (optional)
	int nSolution;			// Number of best solutions output for postprocessor
	int nTargetBreak;		// Number of break points between the lower and upper target values of trade-off curve 
	int nMaxRuns;			// Maximum number of runs (specified in the input file)
	double lfCostLimit;		// cost limit for running option 2 (Maximize Control)
	double lfStopDelta;		// Stop criteria, $ for Minimize Cost, % for Maximize Control (optional)

	COleDateTime startDate;
	COleDateTime endDate;

	int* nSedflag;			// array of sediment flag
	double* polmultiplier;	// array of pollutant multiplier
	POLLUTANT* m_pPollutant;
	BMPCOST* m_pBMPcost;	// array for unique bmp type in the project
	int nPollutant;			// number of pollutants in the time series
	int nNWQ;				// number of pollutants (TSS splits into sand, silt, and clay)

	int m_nNum;
	int nETflag;			// ET calculation flag (0-constant monthly ET,1-daily evaporation rate from the timeseries data,2-calculated from the daily temperature data)
	double lfLatitude;		// Latitude (Degree,Minute,Second)
	double lfmonET[12];		// Constant monthly ET rate (in/day) if ET flag is 0 otherwise monthly coefficient to calculate ET values
	double* m_pDataClimate;	// pointer to the timeseries data for climate file
	CString strClimateFileName;	// climate file path
	int m_nSedQualFlag;		// sediment-associated qual flag

	CObList luList;
	CObList siteluList;
	CObList bmpsiteList;
	CObList routeList;		// routing list
	CObList sitepsList;		// site point source list
	CObList AquiferList;	
	CObList PumpList;	
	CObList FtableList;	

	//optional weather data
	long lRecords,lStartIndex,lEndIndex;
	int nWetPeriod;
	int nWetInt;
	double lfWetDays;
	COleDateTime *pWetPeriod;
	WEATHERDATA	*pWEATHERDATA;	// array of original weather data

	//optional
	int nBinFile;		// Precip Bin file (optional) 
	int nBins;			// number of bins
	int* nBinIDs;		// array of bin IDs
	BINDATA	*pBINDATA;	// array of bin data
	CString strBinFile;	// bin file path

// Operations
public:
	CBMPSite* FindBMPSite(const CString& strID);
	CLandUse* FindLandUse(int nLuID);
	CAquifer* FindAquifer(int nAquiferID);
	CPUMP* FindPumpCurve(CString strPCurveID);
	CFtable* FindFtable(CString strFtableID);

	// functions for loading BMP data
	bool ReadInputFile(CString strFileName);
	bool ReadFileSection(FILE *fp, int nSection);
	void SkipCommentLine(FILE *fp);
	bool ReadDataLine(FILE *fp, CString& strData);
	bool ReadBestPopFile(int nBestPopId);
	void OutputFileHeaderForTradeOffCurve(FILE* fp);

	bool ReadWeatherFile(CString strFileName);
	bool ReadBinFile(CString strFileName);
	long GetNumberOfRecords(FILE *fp);
	long FindDataIndex(COleDateTime tCurrent);
	bool MarkWetIntervals(COleDateTime tStart,COleDateTime tEnd);

	// functions for associate landuse with BMP site, creating routing network, and load time series data
	void ClearCheckedFlag();
	bool RoutingCycleExist(CBMPSite* pBMPSite);
	void AddRouteNode(CBMPSite* pBMPSite);
	bool PrepareDataForModel();
	bool ProcessPreDevFlowDurationCurve();

	// functions for outputing calculation results
	bool OpenTimeseriesOutputFiles(const CString& runID);
	void WriteTimeseriesFileHeader(FILE *fp, int NWQ);
	bool CloseTimeseriesOutputFiles();
	bool ProcessPollutantData();
	bool ProcessTransportData();
	double GetWettedPerimeter(double WaterDepth1,double WaterDepth2,
							  double SurfaceArea1,double SurfaceArea2,
							  double FlowLength,double WettedPerimeter1);

	bool LoadClimateTSData(COleDateTime startDate,COleDateTime endDate);
	// helper function for testing and debuging
	CString GetRoutingOrder();
};

bool CheckFilePath(CString strFilePath);

//SWMM5
void InitializeGAInfil(int nNum);
void CopyGAInfil(TGrnAmpt* Source, TGrnAmpt* Target);
void InitializeHortonInfil(int nNum);
void CopyHortonInfil(THorton* Source, THorton* Target);
void InitializeLinkConduitTransect(int nNum);
void CopyLink(TLink* Source, TLink* Target, int nPollutant);
void CopyConduit(TConduit* Source, TConduit* Target);
void CopyTransect(TTransect* Source, TTransect* Target);
void Validate_Conduit(int nNum);
/////////////////////////////////////////////////////////////////////////////

//{{AFX_INSERT_LOCATION}}
// Microsoft Visual C++ will insert additional declarations immediately before the previous line.

#endif // !defined(AFX_BMPDATA_H__FFC3A527_E06A_4F89_83DA_449E3C851C65__INCLUDED_)
