// BMPSite.h: interface for the CBMPSite class.
//
//////////////////////////////////////////////////////////////////////
#include <queue>
using namespace std;

#if !defined(AFX_BMPSITE_H__D3CC98A1_98BD_4E1A_BDD9_42084729FD84__INCLUDED_)
#define AFX_BMPSITE_H__D3CC98A1_98BD_4E1A_BDD9_42084729FD84__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#define	CLASS_A 1
#define	CLASS_B 2
#define	CLASS_C 3		
#define	CLASS_D 4		
#define	CLASS_E 5		
#define	CLASS_X 100
#define INACRE2CF		3629.971	//in-acre to ft3
#define LBpCFT2MGpL		16018.46	//lb/ft3 to mg/l

//SWMM5
#include "SWMM5/headers.h"

class CLandUse;	
class CAquifer;
class CPUMP;	
class CFtable;	

//pump
typedef struct tag_PUMP_CONTROL {
	int m_nPumpFlag;			// pump option (0-NO , 1-YES)
	bool m_bIsPumping;			// pumping status (0-false, 1-true) 
	double m_lfDepth_ON;		// depth (ft) at which pump is started
	double m_lfDepth_OFF;		// depth (ft) at which pump is stopped
	CString m_strPCurveID;		// unique pump curve identifier (continuous string)
} PUMP_CONTROL;

typedef struct tag_ADJUSTABLE_PARAM {
	CString m_strVariable;
	double	m_lfFrom;
	double	m_lfTo;
	double	m_lfStep;
} ADJUSTABLE_PARAM;

typedef struct tag_COST_PARAM {
	double  m_lfLinearCost;				//Cost per unit length of the BMP structure ($/ft)
	double  m_lfAreaCost;               //Cost per unit area of the BMP structure ($/ft^2)
	double  m_lfTotalVolumeCost;        //Cost per unit total volume of the BMP structure ($/ft^3)
	double  m_lfMediaVolumeCost;        //Cost per unit volume of the soil media ($/ft^3)
	double  m_lfUnderDrainVolumeCost;   //Cost per unit volume of the under drain structure ($/ft^3)
	double  m_lfConstantCost;           //Constant cost ($)
	double  m_lfPercentCost;            //Cost in percentage of all other cost (%)
	double  m_lfLengthExp;
	double  m_lfAreaExp;
	double  m_lfTotalVolExp;
	double  m_lfMediaVolExp;
	double  m_lfUDVolExp;
} COST_PARAM;

typedef struct tag_EVALUATION_FACTOR {
	CString m_strFactor;	// Evaluation factor name, e.g. FlowVolume or TSSLoad
	int		m_nFactorGroup;	// -1 for flow, positive number for pollutant column order
	int		m_nFactorType;	// -1 = AAFV, -2 = PDF, -3 = FEF, -4 = FDC, 1 = AAL, 2 = AAC, 3 = MAC, 4 = CEF
	int		m_nCalcMode;	// 1 = %, 2 = Scale, 3 = Value
	int		m_nCalcDays;	// For factor type MAC only, Maxmimum #Days
	int		m_nInterEvent;	// For factor type FEF only, an inter-event duration to define the start and end of an overflow event (hr)
	double  m_lfThreshold;	// For factor type FEF only, Flow Threshold
	double  m_lfLowFlowLimit; // For factor type FDC only, Low Flow Limit (cfs)
	double  m_lfHighFlowLimit;// For factor type FDC only, High Flow Limit (cfs)
	double  m_lfConcThreshold;// For factor type CEF only, Conc Threshold (optional)
	double	m_lfTarget;		// Target value of evaluation factor
	double	m_lfNextTarget;	// Target value of evaluation factor for next target break
	double	m_lfPriorFactor;// Priority factor for maximize control option
	double	m_lfLowerTarget;// Lower target value (for Option Trade-Off Curve only)
	double	m_lfUpperTarget;// Upper target value (for Option Trade-Off Curve only)
	double	m_lfInit;		// Initial value for evaluation factor for the first run
	double	m_lfCurrent;	// Current value for evaluation factor for the optimization run
	double	m_lfPreDev;		// PreDeveloped value for evaluation factor (without BMP)
	double	m_lfPostDev;	// PostDeveloped value for evaluation factor (without BMP)
} EVALUATION_FACTOR;

typedef struct tag_HOLTAN_PARAM {
	double	m_lfVegA;
	double	m_lfGrowth[12];		// growth index
} HOLTAN_PARAM;

typedef struct tag_BMP_A {
	double	 m_lfBasinWidth;	// bmp width (ft)
	double	 m_lfBasinLength;	// bmp length (ft)
	double	 m_lfOrificeHeight;	// orifice height
	double	 m_lfOrificeDiameter;
	double   m_lfOrificeCoeff;	// discharge coefficient
	int		 m_nExitType;
	int		 m_nWeirType;
	double	 m_lfWeirHeight;
	double	 m_lfWeirWidth;
	double	 m_lfWeirAngle;
	double	 m_lfPETmultiplier;
	int		 m_nORelease;		
	int		 m_nPeople;			
	int		 m_nDays;			
	double   m_lfWatDep_i;		// initial water depth (ft)
	double   m_lfTheta_i;		// initial soil moisture (ft/ft)
	double	 m_lfRelease[24];	// hourly water release in a day
	TGrnAmpt m_pGAInfil;		// manage parameters for green ampt equation
	THorton  m_pHortonInfil;	// manage parameters for Horton equation
	PUMP_CONTROL m_pPumpControl;// pump controls
	CPUMP*   m_pPumpCurve;		// Pump Curve, the actual pointer type is CPUMP
} BMP_A;

typedef struct tag_BMP_B {
	double	 m_lfBasinWidth;
	double	 m_lfBasinLength;
	double	 m_lfMaximumDepth;
	double	 m_lfSideSlope1;
	double	 m_lfSideSlope2;
	double	 m_lfSideSlope3;
	double	 m_lfManning;
	double   m_lfPETmultiplier;
	double   m_lfWatDep_i;		// initial water depth (ft)
	double   m_lfTheta_i;		// initial soil moisture (ft/ft)
	TGrnAmpt m_pGAInfil;		// manage parameters for green ampt equation
	THorton  m_pHortonInfil;	// manage parameters for Horton equation
} BMP_B;

typedef struct tag_Conduit {	// conduit input parameters
	int			m_nIndex;		// sequence number	
	CString		m_strID;		// BMP site ID
	CString		m_strCondType;	// Type of cross-section
	CString		m_strCondName;	// for irregular shape only
	TLink		m_pTLink;		// link parameters
	TConduit	m_pTConduit;	// conduit parameters
	TTransect	m_pTTransect;	// transect parameters
} BMP_C;

typedef struct tag_SEGMENT_D {
	int			m_nSegmentID;	
	double		m_lfSX;	
	double		m_lfRNA;	
	double		m_lfSOA;	
} SEGMENT_D;

typedef struct tag_POLLUTT_D {
	double		m_lfQUALSED_frac;	
	double		m_lfQUALDECAY_ads;	
	double		m_lfQUALDECAY_dis;
	double		m_lfTEMPCORR_ads;
	double		m_lfTEMPCORR_dis;
} POLLUTT_D;

typedef struct tag_BMP_D {
	CString	 m_strID;			// BMP ID			
	int      m_nPolRemMethod;	// pollutant removal method (0-1st order decay, 1-Kadlec and Knight method)
	float    m_lfN;				// Manning's n
	float    m_lfdStore;		// depression storage (ft)
	float    m_lfalpha;			// overland flow factor
	float    m_lfinflow;		// inflow rate (ft/sec)
	float    m_lfrunoff;		// runoff rate (ft/sec)
	float    m_lfdepth;			// depth of surface runoff (ft)
	float    m_lflosses;		// evap. + infil. loss rate (ft/sec)
	double	 m_lfBufferWidth;	// Buffer width perpendicular to the surface runoff (ft)
	double	 m_lfFlowLength;	// Buffer length along the surface runoff (ft), a decision variable
	double   m_lfPETmultiplier;	// multiplier to reflect the vegetation factor
	double   m_lfWatDep_i;		// initial water depth (ft)
	double   m_lfTheta_i;		// initial soil moisture (ft/ft)
	double   m_lfOverlandSlope;	// overland slope (ft/ft)
	TGrnAmpt m_pGAInfil;		// manage parameters for green ampt equation
	THorton  m_pHortonInfil;	// manage parameters for Horton equation
} BMP_D;

typedef struct tag_BMP_E {
	CString	 m_strID;				// BMP ID			
	int      m_nPolRemMethod;		// pollutant removal method (0-1st order decay, 1-Kadlec and Knight method)
	float    m_lfN;					// Manning's n
	float    m_lfdStore;			// depression storage (ft)
	float    m_lfalpha;				// overland flow factor
	float    m_lfinflow;			// inflow rate (ft/sec)
	float    m_lfrunoff;			// runoff rate (ft/sec)
	float    m_lfdepth;				// depth of surface runoff (ft)
	float    m_lflosses;			// evap. + infil. loss rate (ft/sec)
	float	 m_lfsArea;				// BMP surface arae (ft2)
	float	 m_lfSatInfil;			// saturated infiltration rate (ft/sec)
	double   m_lfDCIA;				// percent of directlt connected impervious area (0-100), a decision variable
	double	 m_lfFlowLength;		// Flow length (ft)
	double   m_lfOverlandSlope;		// overland slope (ft/ft)
	double	 m_lfTotImpArea;		// Total Impervious Area (acre)
} BMP_E;

typedef struct tag_SEDIMENT 
{
	double m_lfBEDWID;		// Bed width (ft)
	double m_lfBEDDEP;		// Initial bed depth (ft)
	double m_lfBEDPOR;		// Bed sediment porosity
	double m_lfSAND_FRAC;	// Bed sediment sand fraction
	double m_lfSILT_FRAC;	// Bed sediment silt fraction
	double m_lfCLAY_FRAC;	// Bed sediment clay fraction
} SEDIMENT;

typedef struct tag_SAND 
{
	double m_lfD;		// Effective diameter of the transported sand particles (in)
	double m_lfW;		// The corresponding fall velocity in still water (in/sec)
	double m_lfRHO;		// The density of the sand particles (lb/ft3)
	double m_lfKSAND;	// The coefficient in the sandload power function formula
	double m_lfEXPSND;	// The exponent in the sandload power function formula
} SAND;

typedef struct tag_SILTCLAY 
{
	double m_lfD;		// Effective diameter of the transported silt/clay particles (in)
	double m_lfW;		// The corresponding fall velocity in still water (in/sec)
	double m_lfRHO;		// The density of the silt/clay particles (lb/ft3)
	double m_lfTAUCD;	// The critical bed shear stress for deposition (lb/ft2)
	double m_lfTAUCS;	// The critical bed shear stress for scour (lb/ft2)
	double m_lfM;		// The erodibility coefficient of the sediment (lb/ft2/day)
} SILTCLAY;

class TradeOffCurve 
{
public:
	int m_nID;				// Break Point ID
	int m_nQualNum;
	long m_nTSNum;
	double m_lfMult;		// The multiplier to the timeseries file
	double m_lfCost;		// The cost associated for the timeseries file
//	double m_lfSand;		// The fraction of total sediemnt which is sand
//	double m_lfSilt;		// The fraction of total sediemnt which is silt
//	double m_lfClay;		// The fraction of total sediemnt which is clay
	double* m_pDataBrPt;	// pointer to the point source timeseries data 
	CString m_strBrPtFile;	// The timeseries file name
	COleDateTime m_tmStart;

	TradeOffCurve()
	{
		m_nID  = 0;
		m_nQualNum = 0;
		m_nTSNum = 0;
		m_lfMult = 0.0;
		m_lfCost = 0.0;
//		m_lfSand = 0.0;
//		m_lfSilt = 0.0;
//		m_lfClay = 0.0;
		m_strBrPtFile = "";
		m_tmStart = COleDateTime(1890,1,1,0,0,0);
		m_pDataBrPt = NULL;
	}

	virtual ~TradeOffCurve()
	{
		if(m_pDataBrPt != NULL)
			delete []m_pDataBrPt;
	}
};

class POLLUT_RAConc
{
public:
	int 	m_nRDays;				// maximum # of running days
	double*	m_lfRFlow;				// user specified # of running day flow volume (ft3)
	double*	m_lfRLoad;				// user specified # of running day load (lbs)

	//optional
	double  m_lfThreshConc;			// threshold conc. (Count/L for bacteria, lb/L for other pollutants)
	queue<double> qMass;			// holding 24 previous values for mass (lb)

	POLLUT_RAConc()
	{
		m_nRDays  = 0;
		m_lfRFlow = NULL;
		m_lfRLoad = NULL;
		m_lfThreshConc = 0.0;
	}

	virtual ~POLLUT_RAConc()
	{
		if(m_lfRFlow != NULL)
			delete []m_lfRFlow;
		if(m_lfRLoad != NULL)
			delete []m_lfRLoad;
		while (!qMass.empty())
			qMass.pop();
	}
};

class CBMPSite : public CObject  
{
public:
	CBMPSite();
	CBMPSite(const CString& strID,const CString& strName,const CString& strType,int bmpClass);
	virtual ~CBMPSite();

public:
	CString	m_strID;			// BMP id
	CString	m_strName;			// BMP name
	CString	m_strType;			// BMP type
	bool    m_bChecked;			// used for cycle checking and routing list creation
	bool	m_bUndSwitch;		// switch to turn on the underdrain option (0-off, 1-on)
	int		m_nInfiltMethod;	// Infiltration Method (0-Holtan, 1-Green-Ampt)	
	int		m_nPolRotMethod;	// Pollutant Routing Method (1-Completely mixed, >1-number of CSTRs in series)	
	int		m_nPolRemMethod;	// Pollutant Removal Method (0-1st order decay, 1-kadlec and knight method )	
	int		m_nInfil_Index;		// Index for the Green-Ampt or Horton Infiltration array (positive number)
	int		m_nBMPClass;		// 1-class A; 2-class B; 3-class C; 4-class D; 100-Assessment Point
	int		m_nInterEvent;		// An inter-event duration to define the start and end of an overflow event (hr)
	double	m_lfBMPUnit;		// number of BMP units	
	double	m_lfDDarea;			// BMP design drainage area (acre)
	double	m_lfAccDArea;		// accumulative drainage area for this site (acre)
	double	m_lfPerArea;		// pervious drainage area for this site (acre)
	double	m_lfImpArea;		// impervious drainage area for this site (acre)
	double	m_lfSoilDepth;		// BMP substrate soil depth (ft)
	double	m_lfPorosity;		// Soil porosity (fraction)
	double	m_lfFCapacity;		// Soil field capacity (fraction)
	double	m_lfWPoint;			// Soil wilting point (fraction)
	double	m_lfInfilt;			// Saturated soil infiltration rate (in/hr)
	double	m_lfUndDepth;		// Depth of filter media for underdrain option (ft)
	double	m_lfUndVoid;		// Filter media voids for the underdrain option (fraction)
	double	m_lfUndInfilt;		// Soil infiltration rate underneath the filter media (in/hr)
	double	m_lfCost;			// cost ($)				
	double	m_lfSurfaceArea;	// BMP surface area (acre)				
	double	m_lfExcavatnVol;	// BMP excavation volume (acre-ft)				
	double	m_lfSurfStorVol;	// BMP surface storage volume (acre-ft)				
	double	m_lfSoilStorVol;	// BMP soil storage volume (acre-ft)				
	double	m_lfUdrnStorVol;	// BMP surface storage volume (acre-ft)				

	double  m_lfThreshFlow;		// user defined threshold flow (cfs)
	double  m_lfLowFlowLimit;	// For factor type FDC only, Low Flow Limit (cfs)
	double  m_lfHighFlowLimit;	// For factor type FDC only, High Flow Limit (cfs)
	double	m_lfSiteDArea;		// drainage area for this site (acre)
	double*	m_pDecay;			// decay/loss rate for pollutants
	double*	m_pK;				// Constant rate for pollutants (ft/hr)
	double*	m_pCstar;			// Background concentration for pollutants (lb/ft3)
	double*	m_pConc;			// CSTR concentration for pollutants (lb/ft3)
	double*	m_pUndRemoval;		// removal rate for underdrain pollutants
	void*	m_pSiteProp;		// the actual pointer type could be BMP_A, BMP_B, ..., BMP_X
	CLandUse* m_preLU;			// pre-developed landuse, the actual pointer type is CLanduse
	CAquifer* m_pAquifer;		// aquifer, the actual pointer type is CAquifer
	POLLUT_RAConc* m_RAConc;	// structure of running average concentration
	FILE*    m_fileOut;			// used for outputing simulation result
	HOLTAN_PARAM m_holtanParam;	// manage parameters for holtan equation
	COST_PARAM   m_costParam;	// manage parameters for cost functions
	CPtrList m_dsbmpsiteList;	// element type in the list is DS_BMPSITE
	CPtrList m_adjustList;		// element type in the list is ADJUSTABLE_PARAM
	CPtrList m_factorList;		// element type in the list is EVALUATION_FACTOR
	CPtrList m_usbmpsiteList;	// list for all upstream BMP sites which flows into this BMP site directly, element type in the list is DS_BMPSITE
	CObList  m_siteluList;		// list for all site land use associated with this BMP site
	CObList  m_sitepsList;		// list for all site point sources associated with this BMP site
	int m_nBreakPoints;			// total number of break points on the cost effectiveness curve
	double m_lfBreakPtID;		// break point ID for the timeseries
	CString  m_strCostFile;		// tradeoff curve solution file
	TradeOffCurve* m_TradeOff;	// array of Break point timeseries files
	int m_nQualNum;				// total number of constituents in the time series including flow and water quality
	long m_nTSNum;				// total number of records for the simulation duration 
	double* m_pDataMixLU;		// pointer to the timeseries data for internal land simulation (mixed landuses)
	double* m_pDataPreLU;		// pointer to the timeseries data for internal land simulation (predeveloped landuses)
	COleDateTime m_tmStart;		// start date and time of the time series data 
	SEDIMENT m_sediment;		// general parameters for sediment transport
	SAND     m_sand;			// sand parameters for sediment transportt
	SILTCLAY m_silt;			// silt/clay parameters for sediment transport
	SILTCLAY m_clay;			// clay parameters for sediment transport
	queue<double> qFlow;		// holding previous day values for flow (cfs)
	
	//Ftable
	int m_nFtableFlag;			// Ftable option (0-NO , 1-YES, 2-STORAGE ONLY)
	CFtable* m_pFtable;			// Ftable, the actual pointer type is CFtable
	
	//flow duration curve
	CString m_strPreDevFile;	// predeveloped file path
	int m_nPreDevFlag;			// Predeveloped flag (1-internal predeveloped timeseries,
								//					  2-external predeveloped timeseries) 
	int m_nNumBins;				// number of bins (if 0 then use every timestep)
	int m_nStartBin;			// starting bin that meet the low flow condition
	int m_nEndBin;				// ending bin that meet the high flow condition
	double m_lfMinFlow;			// minimum flow value for the entire simulation (cfs)
	double m_lfMaxFlow;			// maximum flow value for the entire simulation (cfs)

	int* m_pPreDevBinID;		// array of bin IDs for the predeveloped timeseries
	double* m_pPreDevFlow;		// array of predeveloped flow timeseries (cfs)
	double* m_pPreDevBinCFS;	// array of predeveloped average flow by bins (cfs)

	int* m_pBinID;				// array of bin IDs for the simulated timeseries
	double* m_pFlow;			// array of simulated flow timeseries (cfs)
	double* m_pBinCFS;			// array of simulated average flow by bins (cfs)
	
	FILE *fp_FDC;				// Flow Duration Curve output file

public:
	double* GetVariablePointer(CString strVarName);
	double GetBMPArea();
	bool ReadTradeOffCurveCosts();
	bool LoadTradeOffCurveData(long deltm, int BPindex,COleDateTime startDate, COleDateTime endDate);
	bool UnLoadTradeOffCurveData(int nBrPtIndex);
	bool LoadWatershedTSData(long deltm, CString BMPSiteID, COleDateTime startDate, COleDateTime endDate, 
		 CString strFileName, double* multiplier, int landfg);
	void updatePondedDepth2(float* depth, float* dt);
	void getSubareaRunoff2(float inflow, float precip, float evap, float tStep, float* infil, float* depth, float* runoff);
	bool ReadPreDevFile(int landtimestep,COleDateTime startDate,COleDateTime endDate);
	bool ProcessPreDevData(int landtimestep,COleDateTime startDate,COleDateTime endDate,CString strOutputDir);
	bool ProcessSimRunData(CString strRunMode,int landtimestep,COleDateTime startDate,COleDateTime endDate,CString strOutputDir);
	bool Open_FDC_OutputFiles(CString strRunMode, CString strOutputDir);
	void Write_FDC_FileHeader(CString strRunMode);
	bool Close_FDC_OutputFiles();
};

typedef struct tag_DS_BMPSITE 
{
	int		  m_nOutletType;	// 1-total 2-weir 3-orifica or channel 4-underdrain
	CBMPSite* m_pDSBMPSite;		// pointer to a downstream BMP site
} DS_BMPSITE;

typedef struct tag_US_BMPSITE 
{
	int		  m_nOutletType;	// 1-total 2-weir 3-orifica or channel 4-underdrain
	CBMPSite* m_pUSBMPSite;		// pointer to a upstream BMP site
} US_BMPSITE;

void getDdDt2(float t, float* d, float* dddt);
void getDdDt3(float t, float* d, float* dddt);

#endif // !defined(AFX_BMPSITE_H__D3CC98A1_98BD_4E1A_BDD9_42084729FD84__INCLUDED_)
