// LandUse.cpp: implementation of the CLandUse class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "LandUse.h"
#include "StringToken.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

#define	MAXLINE 1024

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CLandUse::CLandUse()
{
	m_strLanduse = "";
	m_strFileName = "";
	m_nID = 0;
	m_nType = 0;
	m_nQualNum = 0;
	m_nTSNum = 0;
	m_pData = NULL;
	m_tmStart = COleDateTime(1890,1,1,0,0,0);
	m_lfsand_fr = 0.0;
	m_lfsilt_fr = 0.0;
	m_lfclay_fr = 0.0;
}

CLandUse::~CLandUse()
{
	if (m_pData != NULL)
		delete []m_pData;
}

// load the time series data
bool CLandUse::LoadLanduseTSData(long deltm, COleDateTime startDate,COleDateTime endDate,double* multiplier)
{
	int i, j;
	char strLine[MAXLINE];
	CString str;

	FILE *fpin = NULL;
	// open the file for reading
	fpin = fopen (m_strFileName, "rt");
	if(fpin == NULL)
		return false;

    // find the flag "date/time"
	while (!feof(fpin))
	{
		fgets(strLine, MAXLINE, fpin);
		str = strLine;
		str.MakeLower();

		// skip another line once found "date/time"
		if(str.Find("date/time") != -1)
		{
			i = 1;
			while(i-- > 0 && !feof(fpin))
				fgets(strLine, MAXLINE, fpin);
			break;
		}
	}
	
	// count the time series numbers
	COleDateTimeSpan tsSpan = endDate - startDate;
	//m_nTSNum = (long)tsSpan.GetTotalHours() + 24;
	m_nTSNum = (long)tsSpan.GetTotalMinutes()/deltm + 1;
	
    // read first data line for starting date of the time series data
	long nStart = ftell (fpin);
	fgets(strLine, MAXLINE, fpin);
    fseek(fpin, nStart, SEEK_SET);

	int year, month, day, hour, minute;
	str = strLine;
	CStringToken strToken(str);
	strToken.NextToken(); // skip the dummy number
	str = strToken.NextToken();
	year = atoi((LPCSTR)str);

	str = strToken.NextToken();
	month = atoi((LPCSTR)str);

	str = strToken.NextToken();
	day = atoi((LPCSTR)str);
	
	str = strToken.NextToken();
	hour = atoi((LPCSTR)str);

	str = strToken.NextToken();
	minute = atoi((LPCSTR)str);

	if(hour == 24)																	
	{                          
		hour = 0; //it is the beginning of next day
		m_tmStart = COleDateTime(year, month, day, hour, minute, 0) + COleDateTimeSpan(1,0,0,0);
	}
	else
	{
		m_tmStart = COleDateTime(year, month, day, hour, minute, 0);
	}

	tsSpan = startDate - m_tmStart;

	// calculate how many lines we need to skip
	//long nSkipLineNum = (long)tsSpan.GetTotalHours();
	long nSkipLineNum = (long)tsSpan.GetTotalMinutes()/deltm;

    // skip all lines the time stamp is before the specified start date
	while (nSkipLineNum-- > 0 && !feof(fpin))
		fgets(strLine, MAXLINE, fpin);

	if(m_pData != NULL)
		delete []m_pData;
	m_pData = new double[m_nTSNum*m_nQualNum];

    // read the data
	i = 0;
	double *pData = m_pData;

    while (!feof(fpin))
    {
		// read one line
		fgets(strLine, MAXLINE, fpin);

		// get the data
		str = strLine;
		CStringToken strToken1(str);
		strToken1.NextToken(); // serial number
		strToken1.NextToken(); // year
		strToken1.NextToken(); // month
		strToken1.NextToken(); // day
		strToken1.NextToken(); // hour
		strToken1.NextToken(); // minute

		for(j=0; j<m_nQualNum; j++)
		{
			str = strToken1.NextToken();
			if (j > 1 && multiplier != NULL)
				*(pData++) = atof((LPCSTR)str)*multiplier[j-2];	// lb/ac
			else 
				*(pData++) = atof((LPCSTR)str);	//in./ac (Flow and Groundwater recharge) 
		}

		i++;
		if (i == m_nTSNum)
			break;
	}

	fclose(fpin);
	return (i == m_nTSNum);
}
