// Pump.cpp : implementation file
//

#include "stdafx.h"
#include "Pump.h"
#include <math.h>

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CPUMP

CPUMP::CPUMP()
{
	m_strPCurveID = "";
	m_nRecords = 0;
	m_lfDepth = NULL;
	m_lfFlow = NULL;

}

CPUMP::CPUMP(CString strPCurveID, int nRecords, double* lfDepth, double* lfFlow)
{
	m_strPCurveID = strPCurveID;
	m_nRecords = nRecords;
	m_lfDepth = new double[nRecords];
	m_lfFlow = new double[nRecords];
	for (int i=0; i<nRecords; i++)
	{
		m_lfDepth[i] = lfDepth[i];
		m_lfFlow[i] = lfFlow[i];
	}
}

CPUMP::~CPUMP()
{
	if(m_lfDepth != NULL)
		delete []m_lfDepth;
	if(m_lfFlow != NULL)
		delete []m_lfFlow;
}

double CPUMP::table_interpolate(double x, double x1, double y1, double x2, double y2)
//
//  Input:   x = x value being interpolated
//           x1, x2 = x values on either side of x
//           y1, y2 = y values corrresponding to x1 and x2, respectively
//  Output:  returns the y value corresponding to x
//  Purpose: interpolates a y value for a given x value.
//
{
    double dx = x2 - x1;
    if ( fabs(dx) < 1.0e-20 ) return (y1 + y2) / 2.;
    return y1 + (x - x1) * (y2 - y1) / dx;
}

double CPUMP::table_lookup(double x)
//
//  Input:   table = pointer to a TTable structure
//           x = an x-value
//  Output:  returns a y-value
//  Purpose: retrieves the y-value corresponding to an x-value in a table,
//           using interploation if necessary.
//
//  NOTE: if x is below the first table entry, then the first y-value is
//        returned; if x is above the last entry, then the last y-value is
//        returned.
//
{
    double x1,y1,x2,y2;
    //table_getFirstEntry(table, &x1, &y1);
	x1 = m_lfDepth[0];
	y1 = m_lfFlow[0];
    if ( x <= x1 ) return y1;

    //while ( table_getNextEntry(table, &x2, &y2) )
	for (int i=1; i<m_nRecords; i++)
    {
		x2 = m_lfDepth[i];
		y2 = m_lfFlow[i];
        if ( x <= x2 ) 
			return table_interpolate(x, x1, y1, x2, y2);
        x1 = x2;
        y1 = y2;
    }
    return y1;
}